//
//  JPViewController.m
//  Portfolio: GitHub Jobs
//
//  Created by Julius Parishy on 5/2/12.
//  Copyright (c) 2012 DiDev Studios. All rights reserved.
//

#import "GHJobBoardViewController.h"

#import "GHJobListing.h"
#import "GHJobListingView.h"
#import "GHJobListingDetailViewController.h"

@interface GHJobBoardViewController ()

-(void)downloadLatestGitHubJobs;
-(void)addCompanyJobViewsToScrollViewWithJobsArray:(NSArray*)array;

-(void)subscribeToKeyboardNotifications;

-(void)animateOctotocatWithKeyboardInfoDictionary:(NSDictionary*)info;

@end

@implementation GHJobBoardViewController

@synthesize jobGridScrollView;
@synthesize searchTextField;

@synthesize octocatImageView;

@synthesize jobListings;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    // Once the app starts, grab the latest jobs.
    [self downloadLatestGitHubJobs];
    
    // Hook up notifications
    [self subscribeToKeyboardNotifications];
    
    // Set some defaults
    searchIsActive = NO;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}

-(void)animateOctotocatWithKeyboardInfoDictionary:(NSDictionary *)info
{
    
    UIViewAnimationCurve curve = [[info objectForKey:UIKeyboardAnimationCurveUserInfoKey] intValue];
    float duration = [[info objectForKey:UIKeyboardAnimationDurationUserInfoKey] floatValue];
    
    CGRect beginFrame = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue];
    CGRect endFrame = [[info objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
    
    float heightDifference = beginFrame.origin.x - endFrame.origin.x;
    
    [UIView animateWithDuration:duration delay:0.0f options:curve animations:^{
        
        CGRect octocatFrame = self.octocatImageView.frame;
        octocatFrame.origin.y += heightDifference;
        
        self.octocatImageView.frame = octocatFrame;
    } completion:^(BOOL finished) {
        
    }];
}

-(void)subscribeToKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserverForName:UIKeyboardWillShowNotification object:nil queue:[NSOperationQueue mainQueue] usingBlock:^(NSNotification *note) {
        
        [self animateOctotocatWithKeyboardInfoDictionary:note.userInfo];
    }];
    
    [[NSNotificationCenter defaultCenter] addObserverForName:UIKeyboardWillHideNotification object:nil queue:[NSOperationQueue mainQueue] usingBlock:^(NSNotification *note) {
        
        [self animateOctotocatWithKeyboardInfoDictionary:note.userInfo];
        
        if(searchIsActive)
        {
            [self hideSearchView:nil];
        }
    }];
}

-(IBAction)toggleSearchView:(UIButton*)sender
{
    if(searchIsActive)
    {
        [self hideSearchView:sender];
    }
    else
    {
        [self showSearchView:sender];
    }
}

-(IBAction)showSearchView:(UIButton*)sender
{
    // Bring up the keyboard.
    [self.searchTextField becomeFirstResponder];
    
    originalJobGridScrollViewFrame = self.jobGridScrollView.frame;
    
    // Disable button while animating
    if(sender)
        sender.enabled = NO;
    
    [UIView animateWithDuration:0.5f animations:^{
       
        // Get rid of the default grid
        CGRect scrollViewOffscreenFrame = self.jobGridScrollView.frame;
        scrollViewOffscreenFrame.origin.x = -scrollViewOffscreenFrame.origin.x - scrollViewOffscreenFrame.size.width;
        
        self.jobGridScrollView.frame = scrollViewOffscreenFrame;
        
        // Show the search input text field
        self.searchTextField.alpha = 1.0f;
        
    } completion:^(BOOL finished) {
        
        searchIsActive = YES;
        
        // Re-enable button
        if(sender)
            sender.enabled = YES;
    }];
}

-(IBAction)hideSearchView:(UIButton*)sender
{
    // If this was done from IB, we need to hide the keyboard,
    // otherwise it was actived by hiding the keyboard manually
    if([self.searchTextField isFirstResponder])
        [self.searchTextField resignFirstResponder];
    
    // Disable button while animating
    if(sender)
        sender.enabled = NO;
    
    [UIView animateWithDuration:0.5f animations:^{
       
        // Bring back the grid
        self.jobGridScrollView.frame = originalJobGridScrollViewFrame;
        
        // Get rid of the text field.
        self.searchTextField.alpha = 0.0f;
        
    } completion:^(BOOL finished) {
        
        searchIsActive = NO;
        
        // Re-enable button
        if(sender)
            sender.enabled = YES;
    }];
}

-(void)downloadLatestGitHubJobs
{
    NSString* githubJobBaseURLString = @"http://jobs.github.com/positions.json";
    NSURL* githubJobBaseURL = [NSURL URLWithString:githubJobBaseURLString];
    
    NSURLRequest* request = [NSURLRequest requestWithURL:githubJobBaseURL];
    
    [NSURLConnection sendAsynchronousRequest:request queue:[NSOperationQueue mainQueue] completionHandler:^(NSURLResponse* response, NSData* data, NSError* error) {
       
        if(error != nil)
        {
            // Post error here
            NSLog(@"Error: %@", error.localizedDescription);
        }
        else
        {
            NSError* jsonError = nil;
            NSArray* jobs = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:&jsonError];
            
            if(jsonError == nil)
            {
                [self performSelectorOnMainThread:@selector(addCompanyJobViewsToScrollViewWithJobsArray:) withObject:jobs waitUntilDone:NO];
            }
            else
            {
                // Post json error here
                NSLog(@"JSON Error: %@", jsonError);
            }
        }
        
    }];
}

-(void)addCompanyJobViewsToScrollViewWithJobsArray:(NSArray*)jobs
{
    // Some properties for how the jobs are aligned.
    
    float currentRowFramePositionX = 0.0f;
    float currentRowFramePositionY = 0.0f;
    
    const float jobViewWidth = 200.0f;
    const float jobViewHeight = 200.0f;
    const float jobViewGapSize = 20.0f;
    
    // And some grid properties
    
    int numberOfJobs = jobs.count;
    const int jobsPerRow = 4;
    
    int numberOfRows = (numberOfJobs / jobsPerRow);
    if(numberOfJobs % jobsPerRow != 0)
        numberOfRows++;
    
    CGSize contentSize = self.jobGridScrollView.contentSize;
    contentSize.height += (numberOfRows * jobViewHeight) + ((numberOfRows - 1) * jobViewGapSize);
    self.jobGridScrollView.contentSize = contentSize;
    
    float animationDelay = 0.0f;
    
    float animationDelayStep = 0.1f;
    
    for(int i = 0; i < numberOfJobs; ++i)
    {
        NSDictionary* jobDictionary = [jobs objectAtIndex:i];
        
        GHJobListing* listing = [[GHJobListing alloc] initWithDictionary:jobDictionary];
        [self.jobListings addObject:listing];
        
        CGRect jobViewFrame = CGRectMake(currentRowFramePositionX, currentRowFramePositionY, jobViewWidth, jobViewHeight);
        GHJobListingView* jobView = [[GHJobListingView alloc] initWithFrame:jobViewFrame jobListing:listing pressedBlock:^(GHJobListing* listing){
            
            [self showJobListingDetails:listing];
        }];
        
        jobView.alpha = 0.0f;
        [self.jobGridScrollView addSubview:jobView];
        
        [UIView animateWithDuration:0.4f delay:animationDelay options:UIViewAnimationCurveEaseIn animations:^{
            
            jobView.alpha = 1.0f;
            
        } completion:^(BOOL finished) {
            
        }];
        
        animationDelay += animationDelayStep;
        
        currentRowFramePositionX += jobViewWidth + jobViewGapSize;
        
        if((i + 1) % jobsPerRow == 0)
        {
            currentRowFramePositionX = 0.0f;
            currentRowFramePositionY += jobViewHeight + jobViewGapSize;
        }
    }
    
    // Add some flair
    
    UILabel* jpLabel = [[UILabel alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 150.0f, 20.0f)];
    jpLabel.textAlignment = UITextAlignmentCenter;
    CGPoint center = self.jobGridScrollView.center;
    center.y = self.jobGridScrollView.contentSize.height + 150.0f;
    jpLabel.center = center;
    
    jpLabel.backgroundColor = [UIColor clearColor];
    jpLabel.font = [UIFont fontWithName:@"Helvetica Neue" size:12.0f];
    jpLabel.textColor = [UIColor darkGrayColor];
    jpLabel.text = @"made by @jparishy";
    
    [self.jobGridScrollView addSubview:jpLabel];
}

-(void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if([segue.identifier isEqualToString:@"ShowJobListingDetail"] && [sender isKindOfClass:[GHJobListing class]])
    {
        GHJobListingDetailViewController* detail = [segue destinationViewController];
        [detail setJobListing:sender];
    }
}

-(void)showJobListingDetails:(GHJobListing *)listing
{
    [self performSegueWithIdentifier:@"ShowJobListingDetail" sender:listing];
}

@end
